@@ -1,23 +1,12 @@
 {-# LANGUAGE CPP #-}
+{-# LANGUAGE EmptyCase #-}
 {-# LANGUAGE GeneralizedNewtypeDeriving #-}
 {-# LANGUAGE StandaloneDeriving #-}
-{-# LANGUAGE TypeOperators #-}
-#if __GLASGOW_HASKELL__ >= 708
-{-# LANGUAGE EmptyCase #-}
-#endif
-
-#if __GLASGOW_HASKELL__ >= 702
 {-# LANGUAGE Trustworthy #-}
-#endif
-
+{-# LANGUAGE TypeOperators #-}
+{-# OPTIONS_GHC -Wno-deprecations #-}
 {-# OPTIONS_HADDOCK not-home #-}
 
-#if __GLASGOW_HASKELL__ >= 708 && __GLASGOW_HASKELL__ < 710
-{-# OPTIONS_GHC -fno-warn-amp #-}
-#endif
-
-{-# OPTIONS_GHC -fno-warn-deprecations #-}
-
 -----------------------------------------------------------------------------
 -- |
 -- Copyright   :  (C) 2011-2018 Edward Kmett
@@ -37,6 +26,9 @@ module Data.Functor.Bind.Class (
   -- * Wrappers
   , WrappedApplicative(..)
   , MaybeApply(..)
+  , (<.*>)
+  , (<*.>)
+  , traverse1Maybe
   -- * Bindable functors
   , Bind(..)
   , apDefault
@@ -53,12 +45,15 @@ import Control.Arrow
 import Control.Category
 import Control.Monad (ap)
 import Control.Monad.Trans.Cont
-import Control.Monad.Trans.Error
 import Control.Monad.Trans.Except
 import Control.Monad.Trans.Identity
 import Control.Monad.Trans.Maybe
 import Control.Monad.Trans.Reader
-import Control.Monad.Trans.List
+#if MIN_VERSION_transformers(0,5,6)
+import qualified Control.Monad.Trans.RWS.CPS as CPS
+import qualified Control.Monad.Trans.Writer.CPS as CPS
+import Semigroupoids.Internal
+#endif
 import qualified Control.Monad.Trans.RWS.Lazy as Lazy
 import qualified Control.Monad.Trans.State.Lazy as Lazy
 import qualified Control.Monad.Trans.Writer.Lazy as Lazy
@@ -74,28 +69,26 @@ import Data.Bifunctor.Join
 import Data.Bifunctor.Product as Bifunctor
 import Data.Bifunctor.Tannen
 import Data.Bifunctor.Wrapped
+import Data.Complex
 import Data.Functor.Compose
 import Data.Functor.Constant
 import Data.Functor.Identity
 import Data.Functor.Product as Functor
 import Data.Functor.Reverse
 import Data.Functor.Extend
-import Data.List.NonEmpty
+import Data.List.NonEmpty (NonEmpty)
+import Data.Ord (Down (..))
+import Data.Proxy
 import Data.Semigroup as Semigroup
 import qualified Data.Monoid as Monoid
 import Data.Orphans ()
+import GHC.Generics as Generics
 import Language.Haskell.TH (Q)
 import Prelude hiding (id, (.))
 
-#if MIN_VERSION_base(4,6,0)
-import Data.Ord (Down (..))
-#else
-import GHC.Exts (Down (..))
-#endif
-
-
-#if MIN_VERSION_base(4,4,0)
-import Data.Complex
+#if !(MIN_VERSION_transformers(0,6,0))
+import Control.Monad.Trans.Error
+import Control.Monad.Trans.List
 #endif
 
 #ifdef MIN_VERSION_containers
@@ -111,22 +104,12 @@ import Data.Tree (Tree)
 import Data.Tagged
 #endif
 
-#if defined(MIN_VERSION_tagged) || MIN_VERSION_base(4,7,0)
-import Data.Proxy
-#endif
-
 #ifdef MIN_VERSION_unordered_containers
 import Data.Hashable
 import Data.HashMap.Lazy (HashMap)
 import qualified Data.HashMap.Lazy as HashMap
 #endif
 
-#ifdef MIN_VERSION_generic_deriving
-import Generics.Deriving.Base as Generics
-#else
-import GHC.Generics as Generics
-#endif
-
 #ifdef MIN_VERSION_comonad
 import Control.Comonad
 import Control.Comonad.Trans.Env
@@ -177,9 +160,7 @@ class Functor f => Apply f where
   liftF2 f a b = f <$> a <.> b
   {-# INLINE liftF2 #-}
 
-#if defined(__GLASGOW_HASKELL__) && __GLASGOW_HASKELL__ >= 708
   {-# MINIMAL (<.>) | liftF2 #-}
-#endif
 
 #ifdef MIN_VERSION_tagged
 instance Apply (Tagged a) where
@@ -188,12 +169,10 @@ instance Apply (Tagged a) where
   (.>) = (*>)
 #endif
 
-#if defined(MIN_VERSION_tagged) || MIN_VERSION_base(4,7,0)
 instance Apply Proxy where
   (<.>) = (<*>)
   (<.) = (<*)
   (.>) = (*>)
-#endif
 
 instance Apply f => Apply (Backwards f) where
   Backwards f <.> Backwards a = Backwards (flip id <$> a <.> f)
@@ -201,7 +180,7 @@ instance Apply f => Apply (Backwards f) where
 instance (Apply f, Apply g) => Apply (Compose f g) where
   Compose f <.> Compose x = Compose ((<.>) <$> f <.> x)
 
--- | A 'Constant f' is not 'Applicative' unless its 'f' is a 'Monoid', but it is an instance of 'Apply'
+-- | A @'Constant' f@ is not 'Applicative' unless its @f@ is a 'Monoid', but it is an instance of 'Apply'
 instance Semigroup f => Apply (Constant f) where
   Constant a <.> Constant b = Constant (a <> b)
   Constant a <.  Constant b = Constant (a <> b)
@@ -219,7 +198,7 @@ instance (Apply f, Apply g) => Apply (Functor.Product f g) where
 instance Apply f => Apply (Reverse f) where
   Reverse a <.> Reverse b = Reverse (a <.> b)
 
--- | A '(,) m' is not 'Applicative' unless its 'm' is a 'Monoid', but it is an instance of 'Apply'
+-- | A @'(,)' m@ is not 'Applicative' unless its @m@ is a 'Monoid', but it is an instance of 'Apply'
 instance Semigroup m => Apply ((,)m) where
   (m, f) <.> (n, a) = (m <> n, f a)
   (m, a) <.  (n, _) = (m <> n, a)
@@ -241,7 +220,7 @@ instance Apply (Either a) where
   Right _  .> Left a  = Left a
   Right _  .> Right b = Right b
 
--- | A 'Const m' is not 'Applicative' unless its 'm' is a 'Monoid', but it is an instance of 'Apply'
+-- | A @'Const' m@ is not 'Applicative' unless its @m@ is a 'Monoid', but it is an instance of 'Apply'
 instance Semigroup m => Apply (Const m) where
   Const m <.> Const n = Const (m <> n)
   Const m <.  Const n = Const (m <> n)
@@ -272,10 +251,12 @@ instance Apply Maybe where
   (<. ) = (<* )
   ( .>) = ( *>)
 
+#if !(MIN_VERSION_base(4,16,0))
 instance Apply Option where
   (<.>) = (<*>)
   (<. ) = (<* )
   ( .>) = ( *>)
+#endif
 
 instance Apply Identity where
   (<.>) = (<*>)
@@ -295,10 +276,8 @@ instance Arrow a => Apply (WrappedArrow a b) where
   (<. ) = (<* )
   ( .>) = ( *>)
 
-#if MIN_VERSION_base(4,4,0)
 instance Apply Complex where
   (a :+ b) <.> (c :+ d) = a c :+ b d
-#endif
 
 -- Applicative Q was only added in template-haskell 2.7 (GHC 7.4), so
 -- define in terms of Monad instead.
@@ -337,53 +316,69 @@ instance (Hashable k, Eq k) => Apply (HashMap k) where
 instance (Functor m, Monad m) => Apply (MaybeT m) where
   (<.>) = apDefault
 
+#if !(MIN_VERSION_transformers(0,6,0))
 -- ErrorT e is _not_ the same as Compose f (Either e)
 instance (Functor m, Monad m) => Apply (ErrorT e m) where
   (<.>) = apDefault
 
+instance Apply m => Apply (ListT m) where
+  ListT f <.> ListT a = ListT $ (<.>) <$> f <.> a
+#endif
+
 instance (Functor m, Monad m) => Apply (ExceptT e m) where
   (<.>) = apDefault
 
 instance Apply m => Apply (ReaderT e m) where
   ReaderT f <.> ReaderT a = ReaderT $ \e -> f e <.> a e
 
-instance Apply m => Apply (ListT m) where
-  ListT f <.> ListT a = ListT $ (<.>) <$> f <.> a
-
 -- unfortunately, WriterT has its wrapped product in the wrong order to just use (<.>) instead of flap
--- | A 'WriterT w m' is not 'Applicative' unless its 'w' is a 'Monoid', but it is an instance of 'Apply'
+-- | A @'Strict.WriterT' w m@ is not 'Applicative' unless its @w@ is a 'Monoid', but it is an instance of 'Apply'
 instance (Apply m, Semigroup w) => Apply (Strict.WriterT w m) where
   Strict.WriterT f <.> Strict.WriterT a = Strict.WriterT $ flap <$> f <.> a where
     flap (x,m) (y,n) = (x y, m <> n)
 
--- | A 'WriterT w m' is not 'Applicative' unless its 'w' is a 'Monoid', but it is an instance of 'Apply'
+-- | A @'Lazy.WriterT' w m@ is not 'Applicative' unless its @w@ is a 'Monoid', but it is an instance of 'Apply'
 instance (Apply m, Semigroup w) => Apply (Lazy.WriterT w m) where
   Lazy.WriterT f <.> Lazy.WriterT a = Lazy.WriterT $ flap <$> f <.> a where
     flap ~(x,m) ~(y,n) = (x y, m <> n)
 
+#if MIN_VERSION_transformers(0,5,6)
+-- | @since 5.3.6
+instance (Bind m) => Apply (CPS.WriterT w m) where
+  mf <.> mx = mkWriterT $ \w ->
+    unWriterT mf w >>- \(f, w') -> unWriterT (f <$> mx) w'
+#endif
+
 instance Bind m => Apply (Strict.StateT s m) where
   (<.>) = apDefault
 
 instance Bind m => Apply (Lazy.StateT s m) where
   (<.>) = apDefault
 
--- | An 'RWST r w s m' is not 'Applicative' unless its 'w' is a 'Monoid', but it is an instance of 'Apply'
+-- | An @'Strict.RWST' r w s m@ is not 'Applicative' unless its @w@ is a 'Monoid', but it is an instance of 'Apply'
 instance (Bind m, Semigroup w) => Apply (Strict.RWST r w s m) where
   (<.>) = apDefault
 
--- | An 'RWST r w s m' is not 'Applicative' unless its 'w' is a 'Monoid', but it is an instance of 'Apply'
+-- | An @'Lazy.RWST' r w s m@ is not 'Applicative' unless its @w@ is a 'Monoid', but it is an instance of 'Apply'
 instance (Bind m, Semigroup w) => Apply (Lazy.RWST r w s m) where
   (<.>) = apDefault
 
+#if MIN_VERSION_transformers(0,5,6)
+-- | @since 5.3.6
+instance (Bind m) => Apply (CPS.RWST r w s m) where
+  mf <.> mx = mkRWST $ \ r s w ->
+    unRWST mf r s w >>- \(f, s', w') -> unRWST (f <$> mx) r s' w'
+#endif
+
 instance Apply (ContT r m) where
   ContT f <.> ContT v = ContT $ \k -> f $ \g -> v (k . g)
 
 #ifdef MIN_VERSION_comonad
--- | An 'EnvT e w' is not 'Applicative' unless its 'e' is a 'Monoid', but it is an instance of 'Apply'
+-- | An @'EnvT' e w@ is not 'Applicative' unless its @e@ is a 'Monoid', but it is an instance of 'Apply'
 instance (Semigroup e, Apply w) => Apply (EnvT e w) where
   EnvT ef wf <.> EnvT ea wa = EnvT (ef <> ea) (wf <.> wa)
 
--- | A 'StoreT s w' is not 'Applicative' unless its 's' is a 'Monoid', but it is an instance of 'Apply'
+-- | A @'StoreT' s w@ is not 'Applicative' unless its @s@ is a 'Monoid', but it is an instance of 'Apply'
 instance (Apply w, Semigroup s) => Apply (StoreT s w) where
   StoreT ff m <.> StoreT fa n = StoreT ((<*>) <$> ff <.> fa) (m <> n)
 
@@ -415,15 +410,31 @@ instance Alternative f => Alternative (WrappedApplicative f) where
 -- | Transform an Apply into an Applicative by adding a unit.
 newtype MaybeApply f a = MaybeApply { runMaybeApply :: Either (f a) a }
 
+-- | Apply a non-empty container of functions to a possibly-empty-with-unit container of values.
+(<.*>) :: (Apply f) => f (a -> b) -> MaybeApply f a -> f b
+ff <.*> MaybeApply (Left fa) = ff <.> fa
+ff <.*> MaybeApply (Right a) = ($ a) <$> ff
+infixl 4 <.*>
+
+-- | Apply a possibly-empty-with-unit container of functions to a non-empty container of values.
+(<*.>) :: (Apply f) => MaybeApply f (a -> b) -> f a -> f b
+MaybeApply (Left ff) <*.> fa = ff <.> fa
+MaybeApply (Right f) <*.> fa = f <$> fa
+infixl 4 <*.>
+
+-- | Traverse a 'Traversable' using 'Apply', getting the results back in a 'MaybeApply'.
+traverse1Maybe :: (Traversable t, Apply f) => (a -> f b) -> t a -> MaybeApply f (t b)
+traverse1Maybe f = traverse (MaybeApply . Left . f)
+
 instance Functor f => Functor (MaybeApply f) where
   fmap f (MaybeApply (Right a)) = MaybeApply (Right (f     a ))
   fmap f (MaybeApply (Left fa)) = MaybeApply (Left  (f <$> fa))
 
 instance Apply f => Apply (MaybeApply f) where
-  MaybeApply (Right f) <.> MaybeApply (Right a) = MaybeApply (Right (f        a ))
-  MaybeApply (Right f) <.> MaybeApply (Left fa) = MaybeApply (Left  (f    <$> fa))
-  MaybeApply (Left ff) <.> MaybeApply (Right a) = MaybeApply (Left  (($a) <$> ff))
-  MaybeApply (Left ff) <.> MaybeApply (Left fa) = MaybeApply (Left  (ff   <.> fa))
+  MaybeApply (Right f) <.> MaybeApply (Right a) = MaybeApply (Right (f         a ))
+  MaybeApply (Right f) <.> MaybeApply (Left fa) = MaybeApply (Left  (f     <$> fa))
+  MaybeApply (Left ff) <.> MaybeApply (Right a) = MaybeApply (Left  (($ a) <$> ff))
+  MaybeApply (Left ff) <.> MaybeApply (Left fa) = MaybeApply (Left  (ff    <.> fa))
 
   MaybeApply a         <. MaybeApply (Right _) = MaybeApply a
   MaybeApply (Right a) <. MaybeApply (Left fb) = MaybeApply (Left (a  <$ fb))
@@ -461,9 +472,7 @@ instance Apply Monoid.Product where (<.>)=(<*>);(.>)=(*>);(<.)=(<*)
 instance Apply Monoid.Dual where (<.>)=(<*>);(.>)=(*>);(<.)=(<*)
 instance Apply Monoid.First where (<.>)=(<*>);(.>)=(*>);(<.)=(<*)
 instance Apply Monoid.Last where (<.>)=(<*>);(.>)=(*>);(<.)=(<*)
-#if MIN_VERSION_base(4,8,0)
 deriving instance Apply f => Apply (Monoid.Alt f)
-#endif
 -- in GHC 8.6 we'll have to deal with Apply f => Apply (Ap f) the same way
 instance Apply Semigroup.First where (<.>)=(<*>);(.>)=(*>);(<.)=(<*)
 instance Apply Semigroup.Last where (<.>)=(<*>);(.>)=(*>);(<.)=(<*)
@@ -481,7 +490,7 @@ instance (Apply f, Apply g) => Apply (f :.: g) where
 
 instance Apply U1 where (<.>)=(<*>);(.>)=(*>);(<.)=(<*)
 
--- | A 'K1 i c' is not 'Applicative' unless its 'c' is a 'Monoid', but it is an instance of 'Apply'
+-- | A @'K1' i c@ is not 'Applicative' unless its @c@ is a 'Monoid', but it is an instance of 'Apply'
 instance Semigroup c => Apply (K1 i c) where
   K1 a <.> K1 b = K1 (a <> b)
   K1 a <.  K1 b = K1 (a <> b)
@@ -490,11 +499,7 @@ instance Apply Par1 where (<.>)=(<*>);(.>)=(*>);(<.)=(<*)
 
 -- | A 'V1' is not 'Applicative', but it is an instance of 'Apply'
 instance Apply Generics.V1 where
-#if __GLASGOW_HASKELL__ >= 708
   e <.> _ = case e of {}
-#else
-  e <.> _ = e `seq` undefined
-#endif
 
 -- | A 'Monad' sans 'return'.
 --
@@ -526,9 +531,7 @@ class Apply m => Bind m where
   join :: m (m a) -> m a
   join = (>>- id)
 
-#if __GLASGOW_HASKELL__ >= 708
   {-# MINIMAL (>>-) | join #-}
-#endif
 
 returning :: Functor f => f a -> (a -> b) -> f b
 returning = flip fmap
@@ -536,8 +539,8 @@ returning = flip fmap
 apDefault :: Bind f => f (a -> b) -> f a -> f b
 apDefault f x = f >>- \f' -> f' <$> x
 
--- | A '(,) m' is not a 'Monad' unless its 'm' is a 'Monoid', but it is an instance of 'Bind'
-instance Semigroup m => Bind ((,)m) where
+-- | A @'(,)' m@ is not a 'Monad' unless its @m@ is a 'Monoid', but it is an instance of 'Bind'
+instance Semigroup m => Bind ((,) m) where
   ~(m, a) >>- f = let (n, b) = f a in (m <> n, b)
 
 #ifdef MIN_VERSION_tagged
@@ -546,11 +549,9 @@ instance Bind (Tagged a) where
   join (Tagged a) = a
 #endif
 
-#if defined(MIN_VERSION_tagged) || MIN_VERSION_base(4,7,0)
 instance Bind Proxy where
   _ >>- _ = Proxy
   join _ = Proxy
-#endif
 
 instance Bind (Either a) where
   Left a  >>- _ = Left a
@@ -576,8 +577,10 @@ instance Bind IO where
 instance Bind Maybe where
   (>>-) = (>>=)
 
+#if !(MIN_VERSION_base(4,16,0))
 instance Bind Option where
   (>>-) = (>>=)
+#endif
 
 instance Bind Identity where
   (>>-) = (>>=)
@@ -594,6 +597,7 @@ instance Monad m => Bind (WrappedMonad m) where
 instance (Functor m, Monad m) => Bind (MaybeT m) where
   (>>-) = (>>=) -- distributive law requires Monad to inject @Nothing@
 
+#if !(MIN_VERSION_transformers(0,6,0))
 instance (Apply m, Monad m) => Bind (ListT m) where
   (>>-) = (>>=) -- distributive law requires Monad to inject @[]@
 
@@ -603,6 +607,7 @@ instance (Functor m, Monad m) => Bind (ErrorT e m) where
     case a of
       Left l -> return (Left l)
       Right r -> runErrorT (k r)
+#endif
 
 instance (Functor m, Monad m) => Bind (ExceptT e m) where
   m >>- k = ExceptT $ do
@@ -614,20 +619,27 @@ instance (Functor m, Monad m) => Bind (ExceptT e m) where
 instance Bind m => Bind (ReaderT e m) where
   ReaderT m >>- f = ReaderT $ \e -> m e >>- \x -> runReaderT (f x) e
 
--- | A 'WriterT w m' is not a 'Monad' unless its 'w' is a 'Monoid', but it is an instance of 'Bind'
+-- | A @'Lazy.WriterT' w m@ is not a 'Monad' unless its @w@ is a 'Monoid', but it is an instance of 'Bind'
 instance (Bind m, Semigroup w) => Bind (Lazy.WriterT w m) where
   m >>- k = Lazy.WriterT $
     Lazy.runWriterT m >>- \ ~(a, w) ->
     Lazy.runWriterT (k a) `returning` \ ~(b, w') ->
       (b, w <> w')
 
--- | A 'WriterT w m' is not a 'Monad' unless its 'w' is a 'Monoid', but it is an instance of 'Bind'
+-- | A @'Strict.WriterT' w m@ is not a 'Monad' unless its @w@ is a 'Monoid', but it is an instance of 'Bind'
 instance (Bind m, Semigroup w) => Bind (Strict.WriterT w m) where
   m >>- k = Strict.WriterT $
     Strict.runWriterT m >>- \ (a, w) ->
     Strict.runWriterT (k a) `returning` \ (b, w') ->
       (b, w <> w')
 
+#if MIN_VERSION_transformers(0,5,6)
+-- | @since 5.3.6
+instance (Bind m) => Bind (CPS.WriterT w m) where
+  m >>- k = mkWriterT $ \ w ->
+    unWriterT m w >>- \(a, w') -> unWriterT (k a) w'
+#endif
+
 instance Bind m => Bind (Lazy.StateT s m) where
   m >>- k = Lazy.StateT $ \s ->
     Lazy.runStateT m s >>- \ ~(a, s') ->
@@ -638,35 +650,35 @@ instance Bind m => Bind (Strict.StateT s m) where
     Strict.runStateT m s >>- \ ~(a, s') ->
     Strict.runStateT (k a) s'
 
--- | An 'RWST r w s m' is not a 'Monad' unless its 'w' is a 'Monoid', but it is an instance of 'Bind'
+-- | An @'Lazy.RWST' r w s m@ is not a 'Monad' unless its @w@ is a 'Monoid', but it is an instance of 'Bind'
 instance (Bind m, Semigroup w) => Bind (Lazy.RWST r w s m) where
   m >>- k = Lazy.RWST $ \r s ->
     Lazy.runRWST m r s >>- \ ~(a, s', w) ->
     Lazy.runRWST (k a) r s' `returning` \ ~(b, s'', w') ->
       (b, s'', w <> w')
 
--- | An 'RWST r w s m' is not a 'Monad' unless its 'w' is a 'Monoid', but it is an instance of 'Bind'
+-- | An @'Strict.RWST' r w s m@ is not a 'Monad' unless its @w@ is a 'Monoid', but it is an instance of 'Bind'
 instance (Bind m, Semigroup w) => Bind (Strict.RWST r w s m) where
   m >>- k = Strict.RWST $ \r s ->
     Strict.runRWST m r s >>- \ (a, s', w) ->
     Strict.runRWST (k a) r s' `returning` \ (b, s'', w') ->
       (b, s'', w <> w')
 
+#if MIN_VERSION_transformers(0,5,6)
+-- | @since 5.3.6
+instance (Bind m) => Bind (CPS.RWST r w s m) where
+  m >>- k = mkRWST $ \ r s w ->
+    unRWST m r s w >>- \(a, s', w') -> unRWST (k a) r s' w'
+#endif
+
 instance Bind (ContT r m) where
   m >>- k = ContT $ \c -> runContT m $ \a -> runContT (k a) c
 
-{-
-instance ArrowApply a => Bind (WrappedArrow a b) where
-  (>>-) = (>>=)
--}
-
-#if MIN_VERSION_base(4,4,0)
 instance Bind Complex where
   (a :+ b) >>- f = a' :+ b' where
     a' :+ _  = f a
     _  :+ b' = f b
   {-# INLINE (>>-) #-}
-#endif
 
 #ifdef MIN_VERSION_containers
 -- | A 'Map k' is not a 'Monad', but it is an instance of 'Bind'
@@ -702,10 +714,8 @@ instance Bind Monoid.Product where (>>-) = (>>=)
 instance Bind Monoid.Dual where (>>-) = (>>=)
 instance Bind Monoid.First where (>>-) = (>>=)
 instance Bind Monoid.Last where (>>-) = (>>=)
-#if MIN_VERSION_base(4,8,0)
 instance Bind f => Bind (Monoid.Alt f) where
   Monoid.Alt m >>- k = Monoid.Alt (m >>- Monoid.getAlt . k)
-#endif
 -- in GHC 8.6 we'll have to deal with Bind f => Bind (Ap f) the same way
 instance Bind Semigroup.First where (>>-) = (>>=)
 instance Bind Semigroup.Last where (>>-) = (>>=)
@@ -713,11 +723,30 @@ instance Bind Semigroup.Min where (>>-) = (>>=)
 instance Bind Semigroup.Max where (>>-) = (>>=)
 -- | A 'V1' is not a 'Monad', but it is an instance of 'Bind'
 instance Bind Generics.V1 where
-#if __GLASGOW_HASKELL__ >= 708
   m >>- _ = case m of {}
-#else
-  m >>- _ = m `seq` undefined
-#endif
+
+-- | @since 5.3.8
+instance Bind Generics.U1 where (>>-)=(>>=)
+
+-- | @since 5.3.8
+instance Bind f => Bind (Generics.M1 i c f) where
+  M1 m >>- f = M1 $ m >>- \a -> case f a of
+    M1 m' -> m'
+
+-- | @since 5.3.8
+instance Bind m => Bind (Generics.Rec1 m) where
+  Rec1 m >>- f = Rec1 $ m >>- \a -> case f a of
+    Rec1 m' -> m'
+
+-- | @since 5.3.8
+instance Bind Generics.Par1 where
+  Par1 m >>- f = f m
+
+-- | @since 5.3.8
+instance (Bind f, Bind g) => Bind (f :*: g) where
+  m :*: n >>- f = (m >>- fstP . f) :*: (n >>- sndP . f) where
+    fstP (a :*: _) = a
+    sndP (_ :*: b) = b
 
 infixl 4 <<.>>, <<., .>>
 
